
namespace Zipr_SDK
{
typedef std::map<libIRDB::Instruction_t*,Zipr_SDK::RangeAddress_t> InstructionLocationMap_t;

enum ZiprPreference {
	None,
	Must,	
};

class ZiprPluginInterface_t
{
	public:
		virtual void PinningBegin()
		{
		}
		virtual void PinningEnd()
		{
		}
		virtual void DollopBegin()
		{
		}
		virtual void DollopEnd()
		{
		}
		virtual void CallbackLinkingBegin()
		{
		}
		virtual void CallbackLinkingEnd()
		{
		}

		virtual ZiprPreference AddressDollop(const Dollop_t *dollop, const RangeAddress_t &source, Range_t &place, bool &coalesce, bool &fallthrough_ok)
		{
			return None;
		}

		virtual bool WillPluginPlop(libIRDB::Instruction_t*)
		{
			return false;
		}

		virtual size_t InsnSize(libIRDB::Instruction_t*, bool)
		{
			return 0;
		}
		/*
		 * DollopEntryOpeningSize()
		 * DollopEntryClosingSize()
		 *
		 * A plugin that wants to put padding into a
		 * dollop entry before/after an instruction 
		 * should return the size of that padding in 
		 * these functions.
		 */
		virtual size_t DollopEntryOpeningSize(DollopEntry_t*)
		{
			return 0;
		}
		virtual size_t DollopEntryClosingSize(DollopEntry_t*)
		{
			return 0;
		}

		/*
		 * PlopDollopEntry()
		 *
		 * This function is invoked by Zipr for a plugin to 
		 * plop a dollop entry. The entry to plop is given as
		 * the first parameter. The plugin writer can use
		 * it's Place() function to determine the beginning
		 * of the space allocated for this dollop entry. 
		 *
		 * entry: A pointer to the dollop entry to be plopped.
		 *
		 * instruction_address: Because multiple plugins 
		 * may want to plop the same dollop entry, the current 
		 * address of the instruction within the entry is 
		 * passed in this parameter. If the plugin moves that 
		 * instruction, it must update the parameter.
		 *
		 * instruction_size: The size of the instruction itself.
		 *
		 * placed_instruction: A flag that the plugin sets
		 * to tell Zipr whether this plugin placed the
		 * instruction or whether it just wrote into the entry
		 * surrounding the instruction. The plugin should set
		 * this value appropriately. The default is false.
		 *
		 * zipr: Pointer to a Zipr object for plugin access to
		 * functionality provided by zipr.
		 *
		 * As output, the implementation must provide the 
		 * address of the memory that "concludes" the 
		 * dollop entry.
		 */
		virtual RangeAddress_t PlopDollopEntry(Zipr_SDK::DollopEntry_t *entry,
			RangeAddress_t &instruction_address,
			RangeAddress_t &target_address,
			size_t instruction_size,
			bool &placed_instruction)
		{
			return 0;
		}
		virtual std::string ToString()
		{
			return "NamelessPlugin";
		}
		virtual ZiprOptionsNamespace_t *RegisterOptions(ZiprOptionsNamespace_t *)
		{
			return new ZiprOptionsNamespace_t("");
		}

		virtual ZiprPreference RetargetPin(const RangeAddress_t &, const Zipr_SDK::Dollop_t *, RangeAddress_t &)
		{
			return None;
		}
		virtual ZiprPreference RetargetCallback(const RangeAddress_t &, const Zipr_SDK::DollopEntry_t *, RangeAddress_t &)
		{
			return None;
		}
		virtual RangeAddress_t PlaceScoopsBegin(const RangeAddress_t max_addr)
		{
			return max_addr;
		}
		virtual RangeAddress_t PlaceScoopsEnd(const RangeAddress_t max_addr)
		{
			return max_addr;
		}

};

}
